/* $Header: /home/yav/tool/lkf/RCS/lkf.c,v 1.4 1995/10/23 05:50:39 yav Exp $
 * lpr Kanji filer for Canon LIPS-II
 * written by yav (UHD98984@pcvan.or.jp)
 */

#include <stdio.h>

/* jiscode.c */
extern unsigned short ysjis2jis();

static char rcsrev[] = "$Revision: 1.4 $";
static int debug = 0;
static int rawmode = 0;
static int indent = 0;
static int sjis = 1;		/* Shift-JIS input */
#define WIDTH_UNLIMITED -1
static int width = 80;
static char **oargv;
static char *ctlstr[] = {
  "\033n",			/* kanjiin */
  "\017",			/* kanjiout */
  "\033[1m",			/* boldon */
  "\033[0m",			/* boldoff */
  "\033[?25h",			/* bold enable */
  NULL
};

#define PC_KANJIIN 0
#define PC_KANJIOUT 1
#define PC_BOLDON 2
#define PC_BOLDOFF 3
#define PC_BOLDENABLE 4

void usage()
{
  fprintf(stderr, "* lkf[irw] %s *\n", rcsrev);
  fprintf(stderr, "written by yav (UHD98984@pcvan.or.jp)\n");
  fprintf(stderr, "usage: %s -r -iN\n", *oargv);
  fprintf(stderr, "option:\n");
  fprintf(stderr, " -r  raw mode\n");
  fprintf(stderr, " -iN indent (default %d)\n", indent);
}

int rawprt()
{
  int c;
  
  while ((c = getchar()) != EOF)
    putchar(c);
}

#define prtctl(func) fputs(ctlstr[func], stdout)

int prt()
{
  int c, i;
  int kanji, bold, linetop, column;
  unsigned short c0, outcode, outdata;
  unsigned char lastbuf[1024];
  int lastbufc;
  int need_lf;
  
  prtctl(PC_BOLDENABLE);
  memset(lastbuf, 0, sizeof(lastbuf));
  bold = lastbufc = kanji = 0;
  linetop = 1;
  need_lf = column = 0;
  while ((c = getchar()) != EOF) {
    /* line top indent */
    if (linetop) {
      for (i = 0; i < indent; i++)
	putchar(' ');
      linetop = 0;
    }
    outdata = 0;
    switch(c) {
    case '\b':
      --lastbufc;
      --column;
      break;
    case '\n':
      need_lf = 1;
      break;
    case '\t':
      i = 8 - (column % 8);
      column += i;
      while (i--)
	putchar(' ');
      break;
    default:
      if (kanji) {
	outdata = (c0 << 8) | (c & 0xff);
	kanji = 0;
      } else {
	if (c & 0x80) {
	  kanji = 1;
	  c0 = c & 0xff;
	  break;
	} else {
	  outdata = c;
	}
      }
      break;
    }
    if (outdata) {
      if (lastbuf[lastbufc]) {
	bold = outdata == ((outdata & 0xff00) ?
			   (lastbuf[lastbufc] << 8)
			   |lastbuf[(lastbufc+1)%sizeof(lastbuf)]
			   :lastbuf[lastbufc]);
	putchar('\b');
	if (lastbuf[(lastbufc+1)%sizeof(lastbuf)]
	    && !(lastbuf[lastbufc] & 0x80))
	  putchar('\b');
      }
      if (bold)
	prtctl(PC_BOLDON);
      if (outdata & 0xff00) {
	outcode = sjis ? ysjis2jis(outdata) : (outdata & 0x7f7f);
	prtctl(PC_KANJIIN);
	putchar((outcode>>8) & 0xff);
      } else {
	outcode = outdata;
      }
      putchar(outcode & 0xff);
      if (outdata & 0xff00)
	prtctl(PC_KANJIOUT);
      if (bold) {
	prtctl(PC_BOLDOFF);
	bold = 0;
      }
      if (outdata & 0xff00) {
	lastbuf[(lastbufc+sizeof(lastbuf)/2)%sizeof(lastbuf)] = '\0';
	lastbuf[lastbufc++] = outdata >> 8;
	column++;
      }
      lastbuf[(lastbufc+sizeof(lastbuf)/2)%sizeof(lastbuf)] = '\0';
      lastbuf[lastbufc++] = outdata;
      column++;
      if (width != WIDTH_UNLIMITED && column >= width)
	need_lf = 1;
    }
    lastbufc %= sizeof(lastbuf);
    if (need_lf) {
      need_lf = 0;
      putchar('\r');
      putchar('\n');
      lastbufc = 0;
      column = 0;
      linetop = 1;
      memset(lastbuf, 0, sizeof(lastbuf));
    }
  }
  return 0;
}

void set_default()
{
  char *p;

  p = *oargv;
  while (*p)
    p++;
  --p;
  switch (*p) {
  case 'i':
    indent = 16;
    break;
  case 'r':
    rawmode = 1;
    break;
  case 'w':
    width = WIDTH_UNLIMITED;
    break;
  default:
    break;
  }
}

int main(argc, argv)
     int argc; char **argv;
{
  int i;
  char **p;
  
  oargv = argv;
  set_default();
  while (--argc > 0 && **++argv == '-') {
    switch(*++*argv) {
    case '?':
    case 'V':
    case 'v':
      usage();
      return 0;
    case 'd':
      debug = 1;
      break;
    case 'c':			/* if: allow control character */
    case 'r':
      rawmode = 1;
      break;
    case 'w':			/* if: width */
      i = strtol(++*argv, NULL, 0);
      if (i > 0)
	width = i;
      break;
    case 'l':			/* if: length */
      break;
    case 'i':			/* if: indent */
      i = strtol(++*argv, NULL, 0);
      if (i >= 0)
	indent = i;
      break;
    case 'n':			/* if: login name */
    case 'h':			/* if: hostname */
      if (argc) {
	argc--;
	argv++;
      }
      break;
    }
  }
  if (rawmode)
    return rawprt();
  return prt();
}

/* End of file */
